
local Actor = Object:extend()

function Actor:new(x, y)
  self.type = "Actor"
  self.x = x or 0
  self.y = y or 0
  self.width = 32
  self.height = 32
  self.xvel = 200
  self.yvel = 0
  self.color = {255, 255, 255, 255}
  self.isFalling = false
  self.gravity = 800
  self.removed = false
end

-- The "move" function also contains the collision resolving part --
-- This can also be seperated with some more effort, but for the moment its just fine --
function Actor:move(dx, dy)
  local object
  local newx, newy = self.x + dx, self.y + dy
  local collider = {x = newx, y = newy, width = self.width, height = self.height}
  
  -- Basic Version: map:getCollisionBoxes need to get replaced by a function which returns all collidable entities for the moving object like enemies 
  -- and so on. Currently it just check for collision against the solid map tiles.
  for k, other in ipairs(map:getCollisionBoxes()) do
    if boxCollision(collider, other) then
      
      object = other
      
      -- The following code corrects the position of the moving object so that it exactly touches the other object without a gap between them.
      -- It also reset some values if the object got on ground so that it stop falling etc.
      -- Right Side --
      if self.x + self.width < object.x then
        self.x = object.x - self.width

      -- Left Side --
      elseif self.x > object.x + object.width then
        self.x = object.x + object.width
      
      -- On Top --
      elseif self.y + self.height <= object.y then
        self.y = object.y - self.height
        self.yvel = 0
        self.isFalling = false
        
      -- Underneath --
      elseif self.y > object.y + object.height and self.yvel <= 0 then
        self.y = object.y + object.height
        self.yvel = 0
      end
      
    end
  end

  -- if there is no object - move freely :)
  if not object then
    self.x = newx
    self.y = newy
  end
  
end


function Actor:jump(vel)
  if self.isFalling then return end
  self.yvel = vel
  self.isFalling = true
end


function Actor:updateGravity(dt)
  self.isFalling = true
  self.yvel = self.yvel + (self.gravity * dt)
  self:move(0, self.yvel*dt)
end


function Actor:destroy()
  self.removed = true
end


function Actor:update(dt)
end


function Actor:draw()
  love.graphics.setColor(unpack(self.color))
  love.graphics.rectangle("fill", self.x, self.y, self.width, self.height)
  love.graphics.setColor(255, 255, 255, 255)
end

return Actor
